@@ -80,6 +80,8 @@ class GitHandler(object):
         else:
             self.gitdir = self.repo.join('git')
 
+        self.init_author_file()
+
         self.paths = ui.configitems('paths')
 
         self.branch_bookmark_suffix = ui.config('git', 'branch_bookmark_suffix')
@@ -95,6 +97,19 @@ class GitHandler(object):
             os.mkdir(self.gitdir)
             self.git = Repo.init_bare(self.gitdir)
 
+    def init_author_file(self):
+        self.author_map = {}
+        if self.ui.config('git', 'authors'):
+            with open(self.repo.wjoin(
+                self.ui.config('git', 'authors')
+            )) as f:
+                for line in f:
+                    line = line.strip()
+                    if not line or line.startswith('#'):
+                        continue
+                    from_, to = re.split(r'\s*=\s*', line, 2)
+                    self.author_map[from_] = to
+
     ## FILE LOAD AND SAVE METHODS
 
     def map_set(self, gitsha, hgsha):
@@ -352,19 +367,66 @@ class GitHandler(object):
         return commit.id
 
     def get_valid_git_username_email(self, name):
-        return name.lstrip('< ').rstrip('> ')
+        r"""Sanitize usernames and emails to fit git's restrictions.
+
+        The following is taken from the man page of git's fast-import
+        command:
+
+            [...] Likewise LF means one (and only one) linefeed [...]
+
+            committer
+                The committer command indicates who made this commit,
+                and when they made it.
+
+                Here <name> is the person's display name (for example
+                "Com M Itter") and <email> is the person's email address
+                ("cm@example.com[1]"). LT and GT are the literal
+                less-than (\x3c) and greater-than (\x3e) symbols. These
+                are required to delimit the email address from the other
+                fields in the line. Note that <name> and <email> are
+                free-form and may contain any sequence of bytes, except
+                LT, GT and LF. <name> is typically UTF-8 encoded.
+
+        Accordingly, this function makes sure that there are none of the
+        characters <, >, or \n in any string which will be used for
+        a git username or email. Before this, it first removes left
+        angle brackets and spaces from the beginning, and right angle
+        brackets and spaces from the end, of this string, to convert
+        such things as " <john@doe.com> " to "john@doe.com" for
+        convenience.
+
+        TESTS:
+
+        >>> from mercurial.ui import ui
+        >>> g = GitHandler('', ui()).get_valid_git_username_email
+        >>> g('John Doe')
+        'John Doe'
+        >>> g('john@doe.com')
+        'john@doe.com'
+        >>> g(' <john@doe.com> ')
+        'john@doe.com'
+        >>> g('    <random<\n<garbage\n>  > > ')
+        'random???garbage?'
+        >>> g('Typo in hgrc >but.hg-git@handles.it.gracefully>')
+        'Typo in hgrc ?but.hg-git@handles.it.gracefully'
+        """
+        return re.sub('[<>\n]', '?', name.lstrip('< ').rstrip('> '))
 
     def get_git_author(self, ctx):
         # hg authors might not have emails
         author = ctx.user()
 
+        # see if a translation exists
+        if author in self.author_map:
+            author = self.author_map[author]
+
         # check for git author pattern compliance
         regex = re.compile('^(.*?) ?\<(.*?)(?:\>(.*))?$')
         a = regex.match(author)
 
         if a:
-            name = a.group(1)
-            email = a.group(2)
+            name = self.get_valid_git_username_email(a.group(1))
+            email = self.get_valid_git_username_email(a.group(2))
             if a.group(3) != None and len(a.group(3)) != 0:
                 name += ' ext:(' + urllib.quote(a.group(3)) + ')'
             author = self.get_valid_git_username_email(name) + ' <' + self.get_valid_git_username_email(email) + '>'
@@ -535,8 +597,18 @@ class GitHandler(object):
                         'name' in self._tags:
                     # Mercurial 1.5 and later.
                     del self.repo._tags[name]
-                if name in self.repo._tagtypes:
+                if (util.safehasattr(self.repo, '_tagtypes') and
+                    self.repo._tagtypes and
+                    name in self.repo._tagtypes):
+                    # Mercurial 1.9 and earlier.
                     del self.repo._tagtypes[name]
+                elif (util.safehasattr(self.repo, 'tagscache') and
+                      self.repo.tagscache and
+                      util.safehasattr(self.repo.tagscache, '_tagtypes') and
+                      self.repo.tagscache._tagtypes and
+                      name in self.repo.tagscache._tagtypes):
+                    # Mercurial 2.0 and later.
+                    del self.repo.tagscache._tagtypes[name]
 
     def import_git_commit(self, commit):
         self.ui.debug(_("importing: %s\n") % commit.id)
@@ -724,7 +796,7 @@ class GitHandler(object):
             ctx = self.repo[rev]
             if getattr(ctx, 'bookmarks', None):
                 labels = lambda c: ctx.tags() + [
-                                fltr for fltr, bm 
+                                fltr for fltr, bm
                                 in self._filter_for_bookmarks(ctx.bookmarks())
                             ]
             else:
@@ -861,7 +933,7 @@ class GitHandler(object):
                 bms = bookmarks.parse(self.repo)
             else:
                 bms = self.repo._bookmarks
-            return dict([(filtered_bm, hex(bms[bm])) for 
+            return dict([(filtered_bm, hex(bms[bm])) for
                         filtered_bm, bm in self._filter_for_bookmarks(bms)])
         except AttributeError: #pragma: no cover
             return {}
@@ -930,7 +1002,7 @@ class GitHandler(object):
                 real_branch_names = self.repo.branchmap()
                 bms = dict(
                     (
-                        bm_name + self.branch_bookmark_suffix 
+                        bm_name + self.branch_bookmark_suffix
                             if bm_name in real_branch_names
                         else bm_name,
                         bms[bm_name]
@@ -1003,6 +1075,8 @@ class GitHandler(object):
                 if line == '':
                     continue
 
+                if ' : ' not in line:
+                    break
                 command, data = line.split(" : ", 1)
 
                 if command == 'rename':
@@ -1096,38 +1170,28 @@ class GitHandler(object):
         if not issubclass(client.get_ssh_vendor, _ssh.SSHVendor):
             client.get_ssh_vendor = _ssh.generate_ssh_vendor(self.ui)
 
-        for handler, transport in (("git://", client.TCPGitClient),
-                                   ("git@", client.SSHGitClient),
-                                   ("git+ssh://", client.SSHGitClient)):
-            if uri.startswith(handler):
-                # We need to split around : or /, whatever comes first
-                hostpath = uri[len(handler):]
-                if (hostpath.find(':') > 0 and hostpath.find('/') > 0):
-                    # we have both, whatever is first wins.
-                    if hostpath.find(':') < hostpath.find('/'):
-                      hostpath_seper = ':'
-                    else:
-                      hostpath_seper = '/'
-                elif hostpath.find(':') > 0:
-                    hostpath_seper = ':'
-                else:
-                    hostpath_seper = '/'
+        # Test for git:// and git+ssh:// URI.
+        #  Support several URL forms, including separating the
+        #  host and path with either a / or : (sepr)
+        git_pattern = re.compile(
+            r'^(?P<scheme>git([+]ssh)?://)(?P<host>.*?)(:(?P<port>\d+))?'
+            r'(?P<sepr>[:/])(?P<path>.*)$'
+        )
+        git_match = git_pattern.match(uri)
+        if git_match:
+            res = git_match.groupdict()
+            transport = client.SSHGitClient if 'ssh' in res['scheme'] else client.TCPGitClient
+            host, port, sepr, path = res['host'], res['port'], res['sepr'], res['path']
+            if sepr == '/':
+                path = '/' + path
+            # strip trailing slash for heroku-style URLs
+            # ssh+git://git@heroku.com:project.git/
+            if sepr == ':' and path.endswith('.git/'):
+                path = path.rstrip('/')
+            if port:
+                client.port = port
 
-                port = None
-                host, path = hostpath.split(hostpath_seper, 1)
-                if hostpath_seper == '/':
-                    transportpath = '/' + path
-                else:
-                    # port number should be recognized
-                    m = re.match('^(?P<port>\d+)?(?P<path>.*)$', path)
-                    if m.group('port'):
-                        client.port = m.group('port')
-                        port = client.port
-                        transportpath = m.group('path')
-                    else:
-                        transportpath = path
-
-                return transport(host, thin_packs=False, port=port), transportpath
+            return transport(host, thin_packs=False, port=port), path
 
         httpclient = getattr(client, 'HttpGitClient', None)
 
